var widgetAPI = new Common.API.Widget();
var tvKey = new Common.API.TVKeyValue();
var pluginAPI = new Common.API.Plugin();
var nnaviPlugin;

String.prototype.startsWith = function (str){
    return this.slice(0, str.length) == str;
};
String.prototype.endsWith = function(suffix) {
    return this.indexOf(suffix, this.length - suffix.length) !== -1;
};

var Core = {
    /**
     * Алиас для document.getElementById
     * @param {String} id идентификатор элемента
     * @returns {@exp;document@call;getElementById}
     */
    getById: function(id){
        return document.getElementById(id);
    },
    /**
     * Возвращает !! 1 !! элемент с указанным классом.
     * @param {String} cls класс. Например, '.active'
     * @returns {@exp;document@call;querySelector|@exp;@call;jQuery@call;get}
     */
    getByClass: function(cls){
        if(document.querySelector != undefined){
            return document.querySelector(cls);
        }else{
            return jQuery(cls).get(0);
        }
    },
    /**
     * Возвращает 1 элемент с указанным классом у данного родителя
     * @param {DOM Element} parent родитель
     * @param {String} childClass класс. Например, '.active'
     * @returns {@exp;@call;jQuery@pro;children@call;@call;get|@exp;parent@call;querySelector}
     */
    getChildByClass: function(parent, childClass){
        if(document.querySelector != undefined){
            return parent.querySelector(childClass);
        }else{
            return jQuery(parent).children(childClass).get(0);
        }
    },
    /**
     * Добавляет класс элементу
     * @param {DOM Element} element элемент
     * @param {String} cls класс. Например, 'active'
     */
    addClass: function(element, cls){
        if(element.classList){
            element.classList.add(cls);
        }else{
            element.className += ' ' + cls;
        }
    },
    /**
     * Удаляет класс у элемента.
     * @param {DOM Element} element элемент
     * @param {String} cls класс. Например, 'active'.
     */
    removeClass: function(element, cls){
        if(element.classList){
            element.classList.remove(cls);
        }else{
            var regex = new RegExp('(?:\\s|^)' + cls + '(?:\\s|$)');
            element.className = element.className.replace(regex, ' ');
        }
    },
    /**
     * Метод, для установки innerHTML
     * @param {DOM Element} element элемент
     * @param {String} innerHTML что надо вставить
     */
    innerHTML: function(element, innerHTML){
        widgetAPI.putInnerHTML(element, innerHTML);
    },
    /**
     * Функция для парсинга json. Нужно для сохранения совместимости со старыми телевизорами
     * @param {JSON-String} json та строка, котрую нужно распарсить
     * @returns {JSON}
     */
    parseJSON: function(json){
        if(window.JSON){
            return JSON.parse(json);
        }else{
            return eval('('+json+')');
        }
    },
    /**
     * Проверяет есть ли элемент в массиве
     * @param {Array} arr массив, в котором искать
     * @param {Object} obj объект, который искать в массиве
     * @returns {Boolean} содержится ли элемент в массиве
     */
    inArray: function(arr, obj) {
        return (arr.indexOf(obj) !== -1);
    },
    /**
     * Удаляет Dom элемент
     * @param {type} element
     * @returns {undefined}
     */
    remove: function(element){
        element.parentNode.removeChild(element);
    },
    showElement: function(element){
        element.style.display = "block";
    },
    hideElement: function(element){
        element.style.display = "none";
    }
};

var SplashScreen = {
    progress: 0,
    startedChannel: 0,
    show: function(){
        KeyHandler.enableBlock();
//        Core.addClass(document.body, "splashBackground");
        Core.addClass(Core.getById("veryveryroot"), "splashBackground");
        
        jQuery("body").append(
            '<div id="splashScreen">'
                +'<div id="splashScreenLogo"><img src="images/common/SplashScreenLogo.png" /></div>'
                +'<div id="splashScreenProgressBar"><div id="splashScreenProgressBarInside"></div></div>'
            +'</div>'
        );
        
        SplashScreen.setProgress(SplashScreen.progress);
        Main.testAvailability();
    },
            
    hide: function(){
        KeyHandler.disableBlock();
        Core.remove(Core.getById("splashScreen"));
//        Core.removeClass(document.body, "splashBackground");
        Core.removeClass(Core.getById("veryveryroot"), "splashBackground");
    },
            
    setProgress: function(progress){
        SplashScreen.progress = progress;
        jQuery("#splashScreenProgressBarInside").width(progress+'%');
        if(SplashScreen.progress == 100){
            SplashScreen.finalizeStart();
        }
    },
            
    finalizeStart: function(){
        SplashScreen.hide();
        
        // запускаю стартовый канал на просмотр
        Channel.curr_channel_id = SplashScreen.startedChannel.id;
        Player.playChannel(SplashScreen.startedChannel, 1);
        //
        
        Menu.showAndHideMenu();
    },
    
    showAlarmMessage: function(){
        Core.remove(Core.getById("splashScreen"));
        jQuery("body").append(
            '<div id="splashScreenMessage">'
                +'<div id="splashScreenMessageLogo"><img src="images/common/SplashScreenLogo.png" /></div>'
                +'<div class="splashScreenMessageTextPadding">'
                    +'<div>Приложение недоступно в сети Вашего оператора</div><div class="splashScreenMessageTextPadding">Для выхода нажмите Return <img id="splashScreenReturnIcon" src="images/common/help_back.png" /></div>'
                +'</div>'    
            +'</div>'
        );
    }
}

var Main = {
    url: 'http://stb.lanta-net.ru/stb-client-pre/seam/resource/rest/',
    /**
     * Выполняется когда приходит событие onload
     */
    onLoad: function() {
        // Указываю, что пора начать работу виджета
        widgetAPI.sendReadyEvent();
        
         // Инициализирую notifier
        Notifier.init();
        Network.init();

        // Инициализирую плеер
        Player.init();
        
        Time.init();
        
        setInterval(Channel.saveLastViewedChannel, 60 * 1000 * 15);

        Main.onShow();
        
        SplashScreen.show();
    },
    onShow: function(){
        nnaviPlugin = Core.getById('pluginObjectNNavi');
        nnaviPlugin.SetBannerState(2);
        
        Main.registKeys();
    },
    registKeys: function(){
        pluginAPI.registKey(tvKey.KEY_UP);
        pluginAPI.registKey(tvKey.KEY_DOWN);
        pluginAPI.registKey(tvKey.KEY_RIGHT);
        pluginAPI.registKey(tvKey.KEY_LEFT);
        pluginAPI.registKey(tvKey.KEY_ENTER);
        pluginAPI.registKey(tvKey.KEY_RETURN);
        pluginAPI.registKey(tvKey.KEY_CH_UP);
        pluginAPI.registKey(tvKey.KEY_CH_DOWN);
        pluginAPI.registKey(tvKey.KEY_PRECH);
        pluginAPI.registKey(tvKey.KEY_1);
        pluginAPI.registKey(tvKey.KEY_2);
        pluginAPI.registKey(tvKey.KEY_3);
        pluginAPI.registKey(tvKey.KEY_4);
        pluginAPI.registKey(tvKey.KEY_5);
        pluginAPI.registKey(tvKey.KEY_6);
        pluginAPI.registKey(tvKey.KEY_7);
        pluginAPI.registKey(tvKey.KEY_8);
        pluginAPI.registKey(tvKey.KEY_9);
        pluginAPI.registKey(tvKey.KEY_0);
        pluginAPI.registKey(tvKey.KEY_INFO);
        pluginAPI.registKey(tvKey.KEY_YELLOW);
        
        pluginAPI.unregistKey(tvKey.KEY_VOL_UP);
        pluginAPI.unregistKey(tvKey.KEY_VOL_DOWN);
        pluginAPI.unregistKey(tvKey.KEY_MUTE);
        
        // Включаю обработку кнопок
        Core.getById("anchor").focus();
    },
    /**
     * Выполняется, когда документу приходит событие onunload
     */
    onUnload: function() {
        // Обязательно нужно деинициализировать плеер
        Player.deinit();
    },
            
    testAvailability: function(){
        if(Main.availabilityQuery()){
            SplashScreen.setProgress(25);
            ProjectInfo.loadInfo(true);
        }else{
            SplashScreen.showAlarmMessage();
        }
    },
            
    availabilityQuery: function(){
        var result = false;
        jQuery.ajax({
            url: Main.url+"peer/test",
            dataType:"json",
            async:false,
            success:function(data,textStatus, xhr) {
                result = data.response;
            }
        });
        return result;
    }
};

var Time = {
    every_ten_seconds: 0,
    plugin: 0,
    rightTime: false,
    init: function(){
        Time.plugin = Core.getById("pluginTime");

        Time.checkTime();
        
        Time.every_ten_seconds = setTimeout(function(){
            var date = Time.getTime();
            if(date.getHours() == 5 && date.getMinutes() == 58){
                Channel.needRefresh = true;
                Program.needRefresh = true;
            }
            if(date.getHours() == 6 && date.getMinutes() == 0){
                if(Channel.needRefresh){
                    Channel.loadChannels();
                }
                if(Program.needRefresh){
                    Program.loadProgram();
                }
            }
        }, 10000);
    },
    getEpochTime: function(){
        return Time.plugin.GetEpochTime()*1000;
    },
    getTime: function(time) {
        if (time == undefined) {
            return new Date(Time.getEpochTime());
        } else {
            return new Date(time);
        }
    },
    checkTime: function(){
        var time = Time.getTime();
        var realTime = Time.getTimeFromServer((time.getTimezoneOffset()/60));
        
        if(realTime != undefined){
            if(Math.abs(realTime - time) < 86400000){ // 24 часа.
                Time.rightTime = true;
            }else{
                Time.rightTime = false;
                Notifier.logError("Обратите внимание, время на вашем телевизоре выставлено некорректно. Отображение телепрограммы отключено.");
            }
        }else{
            Time.rightTime = true;
        }
    },
    getTimeFromServer: function(tz){
        var time;
        jQuery.ajax({
            url: 'http://json-time.appspot.com/time.json?tz=Etc/GMT'+(tz<0?'-':'+')+Math.abs(tz),
            dataType:"json",
            async:false,
            success:function(data,textStatus, xhr) {
                time = new Date(data.datetime);
            }
        });
        return time;
    }
};

var Channel = {
    favourite_hash: '',
    all_hash: '',
    favourite: [],
    favouriteOnlyAvailable: [],
    list: [],
    onlyAvailable: [],
    porn_locked: true,
    current_channel_id: 0,
    savedChannelId: 0,
    favourite_navigation: false,
    favourite_navigation_index: 0,
    prev_channel_id: -1,
    curr_channel_id: -1,
    needRefresh: false,
    /**
     * Загружает каналы
     */
    loadChannels: function(callback) {
        jQuery.ajax({
            url: Main.url+'channels',
            dataType:"json",
            async:false,
            context: Channel,
            beforeSend: function(){alert("Start load channels");},
            success:function(data,textStatus) {
                Channel.parseChannels(data);
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for load channels = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("Channels loaded. Status = "+textStatus);
                if (callback != undefined && callback && typeof(callback) === "function") {
                    callback();
                }
            }
        });
    },
    /**
     * Вспомогательный метод, который разбирает полученные данные из метода loadChannels
     */
    parseChannels: function(json) {
        // Создаю пустой шаблон для навигации в меню
        Menu.tv[2].elements = {0: [], 1: [], 2: [], 3: [], 4: [], 5: [], 6: [], 7: [], 8: [], 9: [], 10: []};
        // Вспомогательный массив
        var addedChannelsId = [];
        // Добавляю в список каналы из json.all
        for(var i = 0; i < json.all.length; i++){
            Channel.list.push(json.all[i]);
            addedChannelsId.push(json.all[i].id);
        }
        // Добавляю в список те каналы, которые были не добавлены на предыдущем шаге
        for(var i = 0; i < json.favourite.length; i++){
            if(!Core.inArray(addedChannelsId, json.favourite[i].id)){
                Channel.list.push(json.favourite[i]);
            }
        }
        // Добавляю каналы в пункт любимые и в шаблон для навигации по меню
        for(var i = 0; i < Channel.list.length; i++){
            if(Channel.list[i].favourite){
                Channel.favourite.push(Channel.list[i]);
            }
            Menu.tv[2].elements[Channel.list[i].category_id].push(Channel.list[i]);
        }
        // Создаю список только доступных каналов
        for(var i = 0; i < Channel.list.length; i++){
            if(Channel.list[i].available){
                Channel.onlyAvailable.push(Channel.list[i]);
            }
        }
        // Сортирую любимые каналы по favourite_position
        Channel.sortFavouriteChannels();
        // Добавляю любимые каналы для шаблона навигации в меню
        Menu.tv[2].elements[0] = Channel.favourite;
        // Создаю список только доступных любимых каналов
        Channel.refreshOnlyAvailableFavouriteChannels();
        // хэш любимых каналов
        Channel.favourite_hash = json.favourite_hash;
        // хэш остальных каналов
        Channel.all_hash = json.all_hash;
        Channel.needRefresh = false;
    },
    sortFavouriteChannels: function(){
        Channel.favourite.sort(function(a, b){
            if(a.favourite_position < b.favourite_position)
                return -1;
            if(a.favourite_position > b.favourite_position)
                return 1;
            return 0;
        });
    },
    refreshOnlyAvailableFavouriteChannels: function(){
        Channel.favouriteOnlyAvailable = [];
        for(var i = 0; i < Channel.favourite.length; i++){
            if(Channel.favourite[i].available){
                Channel.favouriteOnlyAvailable.push(Channel.favourite[i]);
            }
        }
    },
    /**
     * Проверяет, есть и канал в любимых или нет
     * @param {Integer} id идентификатор канала
     * @returns {Boolean}
     */
    isFavourite: function(id) {
        for(var i = 0; i < Channel.favourite.length; i++){
            if(Channel.favourite[i].id === id){
                return true;
            }
        }
        return false;
    },
    /**
     * Возвращает порядковый номер любимого канала
     * @param {Integer} id идентификатор канала
     * @returns {Number}
     */
    getFavouritePosition: function(id) {
        for(var i = 0; i < Channel.favourite.length; i++){
            if(Channel.favourite[i].id === id){
                return Channel.favourite[i].favourite_position;
            }
        }
        return 0;
    },
    /**
     * Возвращает следующий доступный для просмотра телеканал
     * @returns канал, для просмотра
     */
    getNextChannel: function(){
        if(Channel.favourite_navigation){
            Channel.favourite_navigation = false;
        }
        for(var i = 0; i < Channel.onlyAvailable.length; i++){
            if(Channel.onlyAvailable[i].id == Channel.curr_channel_id){
                if(Channel.porn_locked){
                    if(i != Channel.onlyAvailable.length - 1){
                        for(var j = i+1; j < Channel.onlyAvailable.length; j++){
                            if(Channel.onlyAvailable[j].category_id != 8){
                                Channel.setCurrentChannelId(Channel.onlyAvailable[j].id);
                                return Channel.onlyAvailable[j];
                            }
                        }
                    }
                    Channel.setCurrentChannelId(Channel.onlyAvailable[0].id);
                    return Channel.onlyAvailable[0];
                }else{
                    if(i == Channel.onlyAvailable.length - 1){
                        Channel.setCurrentChannelId(Channel.onlyAvailable[0].id);
                        return Channel.onlyAvailable[0];
                    }else{
                        Channel.setCurrentChannelId(Channel.onlyAvailable[i+1].id);
                        return Channel.onlyAvailable[i+1];
                    }
                }
            }
        }
    },
    /**
     * Возвращает предыдущий доступный для просмотра телеканал
     * @returns канал, для просмотра
     */
    getPrevChannel: function(){
        if(Channel.favourite_navigation){
            Channel.favourite_navigation = false;
        }
        for(var i = 0; i < Channel.onlyAvailable.length; i++){
            if(Channel.onlyAvailable[i].id == Channel.curr_channel_id){
                if(Channel.porn_locked){
                    if(i != 0){
                        for(var j = i-1; j >= 0; j--){
                            if(Channel.onlyAvailable[j].category_id != 8){
                                Channel.setCurrentChannelId(Channel.onlyAvailable[j].id);
                                return Channel.onlyAvailable[j];
                            }
                        }
                    }
                    Channel.setCurrentChannelId(Channel.onlyAvailable[Channel.onlyAvailable.length-1].id);
                    return Channel.onlyAvailable[Channel.onlyAvailable.length-1];
                }else{
                    if(i == 0){
                        Channel.setCurrentChannelId(Channel.onlyAvailable[Channel.onlyAvailable.length-1].id);
                        return Channel.onlyAvailable[Channel.onlyAvailable.length-1];
                    }else{
                        Channel.setCurrentChannelId(Channel.onlyAvailable[i-1].id);
                        return Channel.onlyAvailable[i-1];
                    }
                }
            }
        }
    },
    /**
     * Возвращает следующий любимый доступный для просмотра телеканал
     * @returns канал, для просмотра
     */
    getNextFavouriteChannel: function(){
        if(Channel.favourite_navigation === false){
            Channel.favourite_navigation_index = 0;
            Channel.favourite_navigation = true;
        }else{
            if(Channel.favourite_navigation_index + 1 < Channel.favouriteOnlyAvailable.length){
                Channel.favourite_navigation_index += 1;
            }else{
                Channel.favourite_navigation_index = 0;
            }
        }
        Channel.setCurrentChannelId(Channel.favouriteOnlyAvailable[Channel.favourite_navigation_index].id);
        return Channel.favouriteOnlyAvailable[Channel.favourite_navigation_index];
    },
    /**
     * Возвращает предыдущий любимый доступный для просмотра телеканал
     * @returns канал, для просмотра
     */
    getPrevFavouriteChannel: function(){
        if(Channel.favourite_navigation === false){
            Channel.favourite_navigation_index = 0;
            Channel.favourite_navigation = true;
        }else{
            if(Channel.favourite_navigation_index - 1 >= 0){
                Channel.favourite_navigation_index -= 1;
            }else{
                Channel.favourite_navigation_index = Channel.favouriteOnlyAvailable.length - 1;
            }
        }
        Channel.setCurrentChannelId(Channel.favouriteOnlyAvailable[Channel.favourite_navigation_index].id);
        return Channel.favouriteOnlyAvailable[Channel.favourite_navigation_index];
    },
    /**
     * Возвращает канал.
     * @param {Integer} channelId идентификатор канала
     * @returns {Channel object}
     */
    getChannel: function(channelId){
        for(var i = 0; i < Channel.list.length; i++){
            if(Channel.list[i].id === channelId){
                return Channel.list[i];
            }
        }
    },
    /**
     * Устанавливает новый текущий идентификатор канала
     * @param {Integer} id
     */
    setCurrentChannelId: function(id){
        Channel.prev_channel_id = Channel.curr_channel_id;
        Channel.curr_channel_id = id;
    },
    /**
     * Возвращает канал по номеру
     * @param {Integer} number
     * @returns {unresolved}
     */
    getChannelByNumber: function(number){
        for(var i = 0; i < Channel.list.length; i++){
            if(Channel.list[i].order == number || Channel.list[i].favourite_position == number){
                return Channel.list[i];
            }
        }
    },
    actionFavouriteChannel: function(channel, action){
        jQuery.ajax({
            url: Main.url+'channels/favtoggle?cid='+channel.id,
            dataType:"json",
            async:false,
            beforeSend: function(){alert("Start action favourite channel");},
            success:function(data,textStatus) {
                if(Menu.tv[1].active != 0){
                    if(action == "remove"){
                        for(var i = 0; i < Channel.list.length; i++){
                            if(Channel.list[i].id === channel.id){
                                Channel.list[i].favourite = false;
                                break;
                            }
                        }
                        
                        Channel.favourite = [];
                        for(var i = 0; i < Channel.list.length; i++){
                            if(Channel.list[i].favourite){
                                Channel.favourite.push(Channel.list[i]);
                            }
                        }
                        Channel.sortFavouriteChannels();
                        
                        for(var i = 0; i < Channel.favourite.length; i++){
                            Channel.favourite[i].favourite_position = i+1;
                        }
                        
                        Channel.refreshOnlyAvailableFavouriteChannels();
                    }else{
                        var last = Channel.favourite[Channel.favourite.length-1].favourite_position;
                        for(var i = 0; i < Channel.list.length; i++){
                            if(Channel.list[i].id == channel.id){
                                Channel.list[i].favourite = true;
                                Channel.list[i].favourite_position = last+1;
                                Channel.favourite.push(Channel.list[i]);
                                Channel.sortFavouriteChannels();
                                Channel.refreshOnlyAvailableFavouriteChannels();
                                break;
                            }
                        }
                    }
                    Menu.tv[2].elements[0] = Channel.favourite;
                    Menu.appendChannelsThirdLevel(channel.id, Menu.tv[1].active);
                    Menu.navigationLevel = 2;
                }else{
                    for(var i = 0; i < Channel.list.length; i++){
                        if(Channel.list[i].id === channel.id){
                            Channel.list[i].favourite = false;
                            break;
                        }
                    }

                    Channel.favourite = [];
                    for(var i = 0; i < Channel.list.length; i++){
                        if(Channel.list[i].favourite){
                            Channel.favourite.push(Channel.list[i]);
                        }
                    }
                    Channel.sortFavouriteChannels();

                    for(var i = 0; i < Channel.favourite.length; i++){
                        Channel.favourite[i].favourite_position = i+1;
                    }

                    Channel.refreshOnlyAvailableFavouriteChannels();
                    Menu.tv[2].elements[0] = Channel.favourite;
                    
                    var id = jQuery(".channelActive").prev().data("id");
                    Menu.reinitFavouriteChannelsSecondLevel(Menu.tv[1].active, id);
                    Menu.navigationLevel = 2;
                    Core.removeClass(Core.getByClass('.menuTvProgramActive'), 'menuTvProgramActive');
                    Menu.appendChannelsThirdLevel(id, Menu.tv[1].active);
                }
                Notifier.logSuccess(data.response);
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for action favourite channel = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("Action favourite channel ended. Status = "+textStatus);
            }
        });
    },
    moveFavouriteChannel: function(channel, direction){
        if(direction == 'up'){
            jQuery.ajax({
                url: Main.url+'channels/moveup?cid='+channel.id,
                dataType:"json",
                beforeSend: function(){alert("Start move up favourite channel");},
                success:function(data,textStatus) {
                    var index = 0;
                    for(var i = 0; i < Channel.favourite.length; i++){
                        if(Channel.favourite[i].id == channel.id){
                            index = i;
                        }
                    }
                    var temp = Channel.favourite[index-1].favourite_position;
                    Channel.favourite[index-1].favourite_position = channel.favourite_position;
                    Channel.favourite[index].favourite_position = temp;
                    
                    Channel.sortFavouriteChannels();
                    Channel.refreshOnlyAvailableFavouriteChannels();
                    Menu.moveChannel("up");
                },
                error:function(xhr, ajaxOptions, thrownError){
                    alert("Error during ajax request for move up favourite channel = "+xhr.responseText);
                },
                complete:function(jqXHR, textStatus){
                    alert("Favourite channel moved. Status = "+textStatus);
                }
            });
        }else{
            jQuery.ajax({
                url: Main.url+'channels/movedown?cid='+channel.id,
                dataType:"json",
                beforeSend: function(){alert("Start move down favourite channel");},
                success:function(data,textStatus) {
                    var index = 0;
                    for(var i = 0; i < Channel.favourite.length; i++){
                        if(Channel.favourite[i].id == channel.id){
                            index = i;
                        }
                    }
                    var temp = Channel.favourite[index+1].favourite_position;
                    Channel.favourite[index+1].favourite_position = channel.favourite_position;
                    Channel.favourite[index].favourite_position = temp;
                    
                    Channel.sortFavouriteChannels();
                    Channel.refreshOnlyAvailableFavouriteChannels();
                    Menu.moveChannel("down");
                },
                error:function(xhr, ajaxOptions, thrownError){
                    alert("Error during ajax request for move down favourite channel = "+xhr.responseText);
                },
                complete:function(jqXHR, textStatus){
                    alert("Favourite channel moved. Status = "+textStatus);
                }
            });
        }
    },
    setAspect: function(channelId, aspect){
        for(var i = 0; i < Channel.list.length; i++){
            if(Channel.list[i].id == channelId){
                Channel.list[i].aspect = (aspect=="original"?"letterBox":"stretch");
                break;
            }
        }
        for(var i = 0; i < Channel.favourite.length; i++){
            if(Channel.favourite[i].id == channelId){
                Channel.favourite[i].aspect = (aspect=="original"?"letterBox":"stretch");
                break;
            }
        }
        Channel.refreshOnlyAvailableFavouriteChannels();
        jQuery.ajax({
            url: Main.url+'channels/setaspect?cid='+channelId+'&aspect='+(aspect=="original"?"letterBox":"stretch"),
            dataType:"json",
            beforeSend: function(){alert("Start set aspect");},
            success:function(data,textStatus) {},
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for set aspect = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("Aspect setted. Status = "+textStatus);
            }
        });
    },
    saveLastViewedChannel: function(){
        if(Channel.curr_channel_id != Channel.savedChannelId){
            jQuery.ajax({
                url: Main.url+'channels/setlastviewedchannel?cid='+Channel.curr_channel_id,
                dataType:"json",
                beforeSend: function(){alert("Start saveLastViewedChannel");},
                success:function(data,textStatus) {
                    Channel.savedChannelId = Channel.curr_channel_id;
                },
                error:function(xhr, ajaxOptions, thrownError){
                    alert("Error during ajax request for saveLastViewedChannel = "+xhr.responseText);
                },
                complete:function(jqXHR, textStatus){
                    alert("saveLastViewedChannel. Status = "+textStatus);
                }
            });
        }
    }
};

var Program = {
    last_change: 0,
    program: {},
    currents: {},
    refreshProgramInterval: 0,
    needRefresh: false,
    /**
     * Загружает телепрограмму
     */
    loadProgram: function(callback) {
        jQuery.ajax({
            url: Main.url+'tvProgram/channel/allTvProgrammForDay',
            dataType:"json",
            async:false,
            context: Program,
            beforeSend: function(){alert("Start load Program");},
            success:function(data,textStatus) {
                Program.parseProgram(data);
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for load Program = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("Program loaded. Status = "+textStatus);
                if (callback != undefined && callback && typeof(callback) === "function") {
                    callback();
                }
            }
        });
    },
    /**
     * Разбирает полученную телепрограмму
     * @param {type} json
     */
    parseProgram: function(json) {
        Program.last_change = json.last_change;
        Program.program = json.program;
        Program.currents = {};
        for (var channelId in Program.program) {
            Program.currents[channelId] = 0;
        }
        Program.updateProgramInfo();
        Program.checkProgram();
        Notifier.hideLoading();
        Program.needRefresh = false;
        Core.removeClass(Core.getById('root'), 'unshown');
    },
    /**
     * Возвращает всю телепрограмму для канала
     * @param {type} channel_id id канала
     * @returns {unresolved} телепрограмма, если есть, если нет - undefined
     */
    getAllByChannel: function(channel_id) {
        if (this.program[channel_id] != undefined) {
            return this.program[channel_id];
        } else {
            return undefined;
        }
    },
    /**
     * Возвращает текущую программу для канала
     * @param {type} channel_id
     * @returns {undefined}
     */
    getCurrentByChannel: function(channel_id) {
        var current = this.currentPositionByChannel(channel_id), all = this.getAllByChannel(channel_id), result = undefined;

        if (current != undefined && all != undefined && all[current] != undefined) {
            if (Time.getTime() <= Time.getTime(all[current].timeEnd)) {
                result = all[current];
            } else {
                if (all[current + 1] != undefined) {
                    this.currents[channel_id] = current + 1;
                    result = all[this.currents[channel_id]];
                }
            }
        }
        return result;
    },
    getNextByChannel: function(channel_id) {
        var all = this.getAllByChannel(channel_id), current = this.currentPositionByChannel(channel_id);

        if (all != undefined && current != undefined && all[current + 1] != undefined) {
            return all[current + 1];
        }
        return undefined;
    },
    currentPositionByChannel: function(channelId) {
        if (this.currents[channelId] != undefined) {
            return this.currents[channelId];
        } else {
            return undefined;
        }
    },
    checkProgram: function() {
        var currentIndex, currentByChannel;
        this.refreshProgramInterval = setInterval(function() {
            for (var channel_id in Program.programms) {
                currentIndex = Program.currentPositionByChannel(channel_id);
                if (currentIndex != undefined) {
                    currentByChannel = Program.getAllByChannel(channel_id)[currentIndex];
                    if (currentByChannel != undefined) {
                        if (Time.getTime() >= Time.getTime(currentByChannel.timeEnd)) {
                            Program.currents[channel_id] = currentIndex + 1;
                        }
                    }
                }
            }
            Program.updateProgramInfo();
        }, 60000);
    },
    getProgramPercent: function(program){
        var percent = Math.round((Time.getTime() - Time.getTime(program.timeStart)) * 100 / (Time.getTime(program.timeEnd) - Time.getTime(program.timeStart)));
        if (percent > 100) {
            percent = 100;
        }
        return percent;
    },
    updateProgramInfo: function() {
        if(Time.rightTime){
            if(Menu.navigationLevel > 0 && Menu.zeroLevelActiveId == 'iconTv'){
                var elements = Core.getById("menu2ColumnTvNavigationElements").getElementsByTagName('*');
                for (var i = 0; i < elements.length; i++) {
                    var id = elements[i].getAttribute('data-id'), program = Program.getCurrentByChannel(id);
                    if (program != undefined) {
                        Core.innerHTML(Core.getChildByClass(elements[i], ".channelTvProgram"), '<div class="channelTvProgramTitle">' + program.name + '</div>'
                                + '<div class="channelTvProgramTime">до ' + program.timeEnding + '</div>' // до 14:50
                                + '<div class="channelTvProgramProgress">'
                                + '<div style="width:' + Program.getProgramPercent(program) + '%;" class="channelTvProgramProgressLost"></div>'
                                + '</div>');
                    } else {
                        Core.innerHTML(Core.getChildByClass(elements[i], ".channelTvProgram"), '')
                    }
                }
            }
        }
    }
};

var Notifier = {
    jacked: 0,
    bigBox: 0,
    init: function(){
        Notifier.jacked = humane.create({baseCls: 'humane-jackedup', timeout: 5000});
        Notifier.jacked.success = Notifier.jacked.spawn({addnCls: 'humane-jackedup-success'});
        Notifier.jacked.error = Notifier.jacked.spawn({addnCls: 'humane-jackedup-error'});
        Notifier.jacked.nonHideError = Notifier.jacked.spawn({addnCls: 'humane-jackedup-error', timeout: 0});
        
        Notifier.bigBox = humane.create({baseCls: 'humane-bigbox', timeout: 0});
    },
    logSuccess: function(message){
        Notifier.jacked.success(message);
    },
    logError: function(message){
        Notifier.jacked.error(message);
    },
    logNonHideError: function(message){
        Notifier.jacked.nonHideError(message);
    },
    hideError: function(){
        Notifier.jacked.remove();
    },
    showBuffering: function(){
        Notifier.bigBox.log('<table style="margin: 0 auto;"><tr><td><img src="images/common/loading.gif"/></td><td> Буферизация</td></tr></table>');
    },
    hideBuffering: function(){
        Notifier.bigBox.remove();
    },
    showLoading: function(){
        KeyHandler.enableBlock();
        Notifier.bigBox.log('<table style="margin: 0 auto;"><tr><td><img src="images/common/loading.gif"/></td><td> Загрузка</td></tr></table>');
    },
    hideLoading: function(){
        Notifier.bigBox.remove();
        KeyHandler.disableBlock();
    }
};

var Network = {
    plugin: {},
    internetConnectionInterval: 10000,
    warning_shown: false,
    init: function(){
        Network.plugin = Core.getById('pluginObjectNetwork');
        setInterval(Network.cyclicInternetConnectionCheck, Network.internetConnectionInterval);
//        Network.getGeoIpInfo();
    },
    showWarning: function(){
        Network.warning_shown = true;
        Notifier.logNonHideError("Интернет отсутствует.");
        Core.getById("playerSplashScreen").style.backgroundImage="url('images/video/backgrounds/channel_unavail.jpg')";
        Core.removeClass(Core.getById('playerSplashScreen'), 'unshown');
    },
    hideWarning: function(){
        Network.warning_shown = false;
        Notifier.hideError();
        Core.addClass(Core.getById('playerSplashScreen'), 'unshown');
    },
    cyclicInternetConnectionCheck: function(){
        if(!Network.checkConnection()){
            // Нет подключенного интернета
            if(!Network.warning_shown){
                Network.showWarning();
                Player.stop();
            }
        }else{
            // Интернет есть. Если было показано сообщение, то скрываем его.
            if(Network.warning_shown){
                Player.rePlayChannel();
                Network.hideWarning();
            }
        }
    },
    checkConnection: function() {
        // Определяю тип подключения: сеть или wifi
        var currentInterface = Network.plugin.GetActiveType();

        // Если нет активного подключения
        if (currentInterface === -1) {
            return false;
        }

        // Проверяю кабельное подключение. Если ошибка - возвращаю false
        if (Network.plugin.CheckPhysicalConnection(currentInterface) !== 1) {
            return false;
        }

        // Проверяю HTTP. Если HTTP не доступно, то возвращаю false
        if (Network.plugin.CheckHTTP(currentInterface) !== 1) {
            return false;
        }

        // Всё хорошо.
        return true;
    },
    getGeoIpInfo: function(){
          jQuery.ajax({
            url: "http://freegeoip.net/json/",
            dataType:"json",
            async:false,
            context: Network,
            beforeSend: function(){alert("Start get geoIp Info");},
            success:function(data,textStatus) {
////{"ip":"93.186.103.163","country_code":"RU","country_name":"Russian Federation","region_code":"72","region_name":"Tambovskaya oblast","city":"Tambov","zipcode":"","latitude":52.7317,"longitude":41.4433,"metro_code":"","areacode":""}
                alert(Object.toJSON(data));
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for get geo ip info = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){alert("Geo Ip info received. Status = "+textStatus);}
        });
    }
};

var ProjectInfo = {
    balance: 0,
    blocked: false,
    haveUnpaid: false,
    lastTvProgramChange: 0,
    favourite_hash: 0,
    all_hash: 0,
    hasUnreadMessages: 0,
    countUnreadMessages: 0,
    password: 0,
    lastViewedChannelOrder: 0,
    useProxy: 'false',
    proxyAddress: '',
    loadInfo: function(firstLoad){
        jQuery.ajax({
            url: Main.url+'projectinfo',
            dataType:"json",
            async:false,
            beforeSend: function(){alert("Start load project info");},
            success:function(data,textStatus) {
                ProjectInfo.balance = data.balance;
                if(ProjectInfo.balance < 0){
                    Notifier.logError("У Вас негативный баланс, Вам недоступны некоторые каналы.");
                }
                ProjectInfo.blocked = data.blocked;
                ProjectInfo.haveUnpaid = data.unpaid;
                if(ProjectInfo.blocked){
                    if(ProjectInfo.haveUnpaid){
                        Notifier.logError("Вы были заблокированы (есть неоплаченные рассрочки).");
                    }else{
                        Notifier.logError("Вы были заблокированы.");
                    }
                }
                ProjectInfo.lastTvProgramChange = data.lastTvProgramChange;
                ProjectInfo.favourite_hash = data.favourite_hash;
                ProjectInfo.all_hash = data.all_hash;
                ProjectInfo.hasUnreadMessages = data.hasUnreadMessages;
                ProjectInfo.countUnreadMessages = data.countUnreadMessages;
                if(ProjectInfo.hasUnreadMessages){
                    Notifier.logError("У Вас есть непрочитанные сообщения ("+ProjectInfo.countUnreadMessages+").");
                }
                ProjectInfo.password = data.password;
                ProjectInfo.lastViewedChannelOrder = data.lastViewedChannelId;
                ProjectInfo.useProxy = data.use_proxy;
                ProjectInfo.proxyAddress = decodeURIComponent(data.address_proxy);
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for loadProjectInfo = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("Project info loaded. Status = "+textStatus);
                if(!firstLoad){
                    if(ProjectInfo.favourite_hash != Channel.favourite_hash || ProjectInfo.all_hash != Channel.favourite_hash){
                        var callback = function(){
                            var channel;
                            if(ProjectInfo.lastViewedChannelOrder != undefined && ProjectInfo.lastViewedChannelOrder != null){
                                channel = Channel.getChannelByNumber(ProjectInfo.lastViewedChannelOrder);
                            }else{
                                channel = Channel.getChannel(1);
                            }
                            Channel.curr_channel_id = channel.id;
                            Player.playChannel(channel, 1);
                        }
                        Channel.loadChannels(callback);
                    }
                    if(Program.last_change != ProjectInfo.lastTvProgramChange){
                        Program.loadProgram();
                    }
                }else{
                    SplashScreen.setProgress(50);
                    var callback = function(){
                        SplashScreen.setProgress(75);
                        if(ProjectInfo.lastViewedChannelOrder != undefined && ProjectInfo.lastViewedChannelOrder != null){
                            SplashScreen.startedChannel = Channel.getChannelByNumber(ProjectInfo.lastViewedChannelOrder);
                        }else{
                            SplashScreen.startedChannel = Channel.getChannel(1);
                        }
                        
                        var callback2 = function(){
                            SplashScreen.setProgress(100);
                        }
                        Program.loadProgram(callback2);
                    }
                    Channel.loadChannels(callback);
                }
            }
        });
    },
    remindPassword: function(){
        jQuery.ajax({
            url: Main.url+'projectinfo/remindpass',
            dataType:"json",
            async:false,
            beforeSend: function(){alert("Start remind password");},
            success:function(data,textStatus) {
                Notifier.logSuccess("Сообщение с паролем было отправлено на номер: "+data.phone);
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for remind password = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("Password reminded. Status = "+textStatus);
            }
        });
    },
    checkPassword: function(pass){
        if(pass != ProjectInfo.password){
            Notifier.logError("Пароли не совпадают");
            return false;
        }
        return true;
    }
};

var Messages = {
    readed: [],
    unreaded: [],
    init: function(){
        Messages.loadMessages();
    },
    loadMessages: function(){
        jQuery.ajax({
            url: Main.url+'messages/get',
            dataType: "json",
            async: false,
            context: Messages,
            beforeSend: function(){alert("Start load messages");},
            success:function(data,textStatus) {
                Messages.readed = data.readed;
                Messages.unreaded = data.unreaded;
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for loadMessages = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("Messages loaded. Status = "+textStatus);
            }
        });
    },
    getReadedMessages: function(){
        return Messages.readed;
    },
    getUnreadedMessages: function(){
        return Messages.unreaded;
    },
    showMessage: function(id){
        var message = null, type;
        for(var i = 0; i < Messages.readed.length; i++){
            if(Messages.readed[i].id == id){
                message = Messages.readed[i];
                type = 'readed';
                break;
            }
        }
        if(message == null){
            for(var i = 0; i < Messages.unreaded.length; i++){
                if(Messages.unreaded[i].id == id){
                    message = Messages.unreaded[i];
                    type = 'unreaded';
                    break;
                }
            }
        }
        
        Popup.createConfirmPopup(message.title, message.description, 
            [{type: 'submit', action: function(){
                    if(type == "unreaded"){Messages.makeReadedMessage(message);}
                    Popup.hide()
                },name: "Ок"}, 
                {type: 'close', action: function(){Popup.hide()},name: "Закрыть"}]);
    },
    makeReadedMessage: function(message){
        jQuery.ajax({
            url: Main.url+'messages/make_read?id='+message.id,
            dataType: "json",
            beforeSend: function(){alert("Start make read message");},
            success:function(data,textStatus) {},
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for make read message = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("Make read messages. Status = "+textStatus);
            }
        });
        
        var index = 0;
        for(var i = 0; i < Messages.unreaded.length; i++){
            if(Messages.unreaded[i].id == message.id){
                index = i;
                break;
            }
        }
        Messages.unreaded.splice(index, 1);
        Messages.readed.push(message);
        
        Menu.showMessagesList("messagesUnreaded");
        if(index < Menu.infoElements[3].length - 1){
            Core.addClass(Core.getById(Menu.infoElements[3][index]), 'menuElementActive');
        }else{
            Core.addClass(Core.getById(Menu.infoElements[3][Menu.infoElements[3].length - 1]), 'menuElementActive');
        }
    }
};

var News = {
    list: [],
    load: function(){
        jQuery.ajax({
            url: Main.url+'projectinfo/news',
            dataType: "json",
            async: false,
            beforeSend: function(){alert("Start load news");},
            success:function(data,textStatus) {
                News.list = [];
                for(var i = 0; i < data.length; i++){
                    for(var j = 0; j < data[i].feedNews.length; j++){
                        News.list.push(data[i].feedNews[j]);
                    }
                }
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for load news = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("News loaded. Status = "+textStatus);
            }
        });
    },
    getById: function(id){
        for(var i = 0; i < News.list.length; i++){
            if(News.list[i].id == id){
                return News.list[i];
            }
        }
    }
};

var Calculations = {
    balance: 0,
    credit: false,
    creditValue: 0,
    currentMonth: {
        payments: 0,
        bills: 0
    },
    load: function(){
        jQuery.ajax({
            url: Main.url+'projectinfo/calculations',
            dataType: "json",
            async: false,
            beforeSend: function(){alert("Start load calculations");},
            success:function(data,textStatus) {
                Calculations.balance = data.balance.balanceCount;
                Calculations.credit = data.balance.credit;
                Calculations.creditValue = data.balance.creditValue;
                Calculations.currentMonth.payments = data.currentMonth.payments;
                Calculations.currentMonth.bills = data.currentMonth.bills;
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for load calculations = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("Calculations loaded. Status = "+textStatus);
            }
        });
    }
};

var Subscriptions = {
    list: [],
    load: function(){
        var result;
        jQuery.ajax({
            url: Main.url+'tvTarif',
            dataType: "json",
            async: false,
            beforeSend: function(){alert("Start load tvTarif");},
            success:function(data,textStatus) {
                result = data;
                Subscriptions.list = result;
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for load tvTarif = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("tvTarif loaded. Status = "+textStatus);
            }
        });
        return result;
    },
    getById: function(id){
        for(var i = 0; i < Subscriptions.list.profiles.length; i++){
            if(Subscriptions.list.profiles[i].id == id){
                return Subscriptions.list.profiles[i];
            }
        }
    },
    runActionTVProfile: function(profileId){
        jQuery.ajax({
            url: Main.url+'projectinfo/actiontvprofile?pid='+profileId,
            dataType:"json",
            async:false,
            beforeSend: function(){alert("Start run action tv profile");},
            success:function(data,textStatus) {
                switch(data.result){
                    case 1:
                        Notifier.logSuccess("Отмена отключения от пакета успешна.");
                        Menu.renewSubscriptionMenu();
                        break;
                    case 2:
                        Notifier.logSuccess("Пакет подписан.");
                        Menu.renewSubscriptionMenu();
                        setTimeout(Channel.loadChannels, 4000);
                        break;
                    case 3:
                        Notifier.logSuccess("Пакет будет отключен в следующем месяце.");
                        Menu.renewSubscriptionMenu();
                        break;
                }
            },
            error:function(xhr, ajaxOptions, thrownError){
                alert("Error during ajax request for run action tv profile = "+xhr.responseText);
            },
            complete:function(jqXHR, textStatus){
                alert("Tv profile actioned. Status = "+textStatus);
            }
        });
    }
};