var Player = {
    plugin: null,
    state: -1,
    originalSource: null,
    STOPPED: 0,
    PLAYING: 1,
    PAUSED: 2,
    hidePanelsTimeout: 0,
    panelsShown: false,
    volumeHideTimeout: 0,
    everyTenSecond: 0,
    currentChannel: {},
    channel: 0,
    playChannelTimeout: 0,
    reopenInterval: 0,
    width: 0,
    height: 0,
    currentWindowType: 'fullscreen',
    canChangeVideoregime: false,
    videoregimeTimeout: 0,
    countAudioTracks: -9,
    audioTracksInfo: [],
    currentStreamId: 0,
    audiotrackTimeout: 0,
    videoResolution: '',
    handleVideoregime: false,
    
    realTimePaused: false,
    
    init: function() {
        this.state = this.STOPPED;

        this.plugin = Core.getById('pluginObjectSEF');
        this.plugin.Open('Player', '1.010', 'Player');

        this.plugin.OnConnectionFailed = 'Player.ConnectionFailed';
        this.plugin.OnAuthenticationFailed = 'Player.AuthenticationFailed';
        this.plugin.OnStreamNotFound = 'Player.StreamNotFound';
        this.plugin.OnNetworkDisconnected = 'Player.NetworkDisconnected';
        this.plugin.OnRenderError = 'Player.RenderError';
        this.plugin.OnBufferingStart = 'Player.onBufferingStart';
        this.plugin.OnBufferingProgress = 'Player.onBufferingProgress';
        this.plugin.OnBufferingComplete = 'Player.onBufferingComplete';
        return true;
    },
    deinit: function() {
        if (this.plugin) {
            Player.stop();
        }
    },
    showPanels: function() {
        if (this.panelsShown == false) {
            this.panelsShown = true;
            Core.removeClass(Core.getById('player'), 'unshown');
        }
    },
    showAndHidePanels: function() {
        window.clearTimeout(this.hidePanelsTimeout);
        this.showPanels();
        this.hidePanelsTimeout = window.setTimeout(Player.hidePanels, 7000);
    },
    hidePanels: function() {
        if (Player.panelsShown == true) {
            Player.panelsShown = false;
            Core.addClass(Core.getById('player'), 'unshown');
        }
    },
    setInfo: function(showFavouriteOrders){
        Core.innerHTML(Core.getById("player_title"), Player.currentChannel.name);
        if(showFavouriteOrders){
            Core.innerHTML(Core.getById("player_number"), Player.currentChannel.favourite_position);
        }else{
            Core.innerHTML(Core.getById("player_number"), Player.currentChannel.order);
        }
        Player.everyTenSecond = window.setInterval(Player.updateInfo, 1000);
        Player.updateInfo();
        Player.showAndHidePanels();
    },
    updateInfo: function(){
        var currentProgram = Program.getCurrentByChannel(Player.currentChannel.id);
        if (currentProgram !== undefined && Time.rightTime) {
            Core.innerHTML(Core.getById("player_curr_program_time"), (currentProgram.timeStarting + "-" + currentProgram.timeEnding));
            Core.innerHTML(Core.getById("player_curr_program_text"), currentProgram.name);
            var nextProgram = Program.getNextByChannel(Player.currentChannel.id);
            if (nextProgram !== undefined) {
                Core.innerHTML(Core.getById("player_next_program_time"), (nextProgram.timeStarting + "-" + nextProgram.timeEnding));
                Core.innerHTML(Core.getById("player_next_program_text"), nextProgram.name);
            }else{
                Core.innerHTML(Core.getById("player_next_program_time"), '');
                Core.innerHTML(Core.getById("player_next_program_text"), '');
            }
            Core.getById('pb_inside').style.width = Program.getProgramPercent(currentProgram) + '%';
            Core.removeClass(Core.getById('player_down_program'), 'unshown');
        } else {
            Core.innerHTML(Core.getById("player_curr_program_time"), '');
            Core.innerHTML(Core.getById("player_curr_program_text"), '');
            Core.innerHTML(Core.getById("player_next_program_time"), '');
            Core.innerHTML(Core.getById("player_next_program_text"), '');
            Core.addClass(Core.getById('player_down_program'), 'unshown');
        }
    },
    playChannel: function(channel, menuId) {
        Player.currentChannel = channel;
        
        pluginAPI.setOffScreenSaver();
        
        if (this.state === this.PLAYING) {
            this.stop();
        }
        this.state = this.PLAYING;
        
        Player.setInfo((menuId == 0));
        
        if(ProjectInfo.useProxy == 'false'){
            Player.plugin.Execute("InitPlayer", Player.currentChannel.mod+'://'+Player.currentChannel.address+':'+Player.currentChannel.port);
        }else{
            Player.plugin.Execute("InitPlayer", ProjectInfo.proxyAddress+Player.currentChannel.address+':'+Player.currentChannel.port);
//            Player.plugin.Execute('Play', 'http://193.203.60.1:50611/udp/' + Player.currentChannel.address + ':' + Player.currentChannel.port);
        }
        Player.checkVideoResolution();
        Player.plugin.Execute("SetTotalBufferSize", 60*1024*1024);
        Player.plugin.Execute("SetInitialBufferSize", 50*1024*1024);
        
        if(Player.currentChannel.radio){
            Core.getById("playerSplashScreen").style.backgroundImage="url('images/video/backgrounds/radio.jpg')";
            Core.removeClass(Core.getById('playerSplashScreen'), 'unshown');
        }else{
            Core.addClass(Core.getById('playerSplashScreen'), 'unshown');
        }
        
        Player.plugin.Execute("StartPlayback");
        Player.plugin.OnEvent = 'Player.onPlayerEvent';
    },
    setPlayerVideoSize: function(positionx, positiony, width, height){
        Player.plugin.Execute('SetDisplayArea',positionx, positiony, width, height);
    },
    checkVideoResolution: function(){
        switch(Player.currentChannel.ratio){
            case '16:9': // HD
                Player.canChangeVideoregime = false;
                Player.currentWindowType = 'fullscreen';
                Player.setPlayerVideoSize(0, 0, 960, 540);
                Core.addClass(Core.getById('playerYellowButtonPanel'), 'unshown');
                break;
            case '20:11': // almost full screen
                Player.canChangeVideoregime = false;
                Player.currentWindowType = 'fullscreen';
                Player.setPlayerVideoSize(0, 5, 960, 530);
                Core.addClass(Core.getById('playerYellowButtonPanel'), 'unshown');
                break;
            default:
                Player.canChangeVideoregime = true;
                if(Player.currentChannel.aspect == 'stretch'){
                    Player.currentWindowType = 'fullscreen';
                    Player.setPlayerVideoSize(0, 0, 960, 540);
                }else{
                    Player.currentWindowType = 'original';
                    if(Player.currentChannel.ratio == '15:11'){
                        Player.setPlayerVideoSize(112, 0, 736, 540);
                    }else{
                        Player.setPlayerVideoSize(120, 0, 720, 540);
                    }
                }
                Core.removeClass(Core.getById('playerYellowButtonPanel'), 'unshown');
                break;
        }
    },
    /**
     * Список event'ов:
     * 1: 'CONNECTION_FAILED',
     * 2: 'AUTHENTICATION_FAILED',
     * 3: 'STREAM_NOT_FOUND',
     * 4: 'NETWORK_DISCONNECTED',
     * 5: 'NETWORK_SLOW',
     * 6: 'RENDER_ERROR',
     * 7: 'RENDERING_START',
     * 8: 'RENDERING_COMPLETE',
     * 9: 'STREAM_INFO_READY',
     * 10: 'DECODING_COMPLETE',
     * 11: 'BUFFERING_START',
     * 12: 'BUFFERING_COMPLETE',
     * 13: 'BUFFERING_PROGRESS',
     * 14: 'CURRENT_PLAYBACK_TIME',
     * 15: 'AD_START',
     * 16: 'AD_END',
     * 17: 'RESOLUTION_CHANGED',
     * 18: 'BITRATE_CHANGED',
     * 19: 'SUBTITLE',
     * 20: 'CUSTOM'
     */
    onPlayerEvent: function(e, a) {
        switch (e){
            case 3:
                Player.StreamNotFound();
                break;
            case 6:
                Player.RenderError();
                break;
            case 9:
                Player.OnStreamInfoReady();
                break;
            case 11:
                Player.onBufferingStart();
                break;
            case 12:
                Player.onBufferingComplete();
                break;
            case 13:
                Player.onBufferingProgress(a);
                break;
        }
    },
    rePlayChannel: function(){
        setTimeout(function(){
            Player.playChannel(Player.currentChannel);
        }, 2000);
    },
    stop: function() {
        if (this.state !== this.STOPPED) {
            this.state = this.STOPPED;
            Player.plugin.Execute("Stop");
        } else {
            alert("Ignoring stop request, not in correct state");
        }
    },
    onBufferingStart: function() {
        Notifier.showBuffering();
    },
    onBufferingProgress: function(percent) {
        alert("Buffering:" + percent + "%");
    },
    onBufferingComplete: function() {
        Notifier.hideBuffering();
        Player.plugin.Execute('SetStreamID',1,0);
    },
    showChannelNumber: function(){
        Core.innerHTML(Core.getById("player_paging_number"), Player.channel);
        Core.removeClass(Core.getById('player_paging_number'), 'unshown');
    },
    hideChannelNumber: function(){
        Core.addClass(Core.getById('player_paging_number'), 'unshown');
    },
    getChannelByNumber: function(number){
        window.clearTimeout(Player.playChannelTimeout);
        if(Player.channel.length < 3){
            Player.channel = ''+Player.channel+number;
        }else{
            Player.channel = ''+number;
        }
        Player.showChannelNumber();
        Player.playChannelTimeout = window.setTimeout(function(){
            Player.hideChannelNumber();
            var chan = Channel.getChannelByNumber(parseInt(Player.channel));
            if(chan != undefined){
                Channel.setCurrentChannelId(chan.id);
                Player.playChannel(chan, (Player.channel.length<3?0:1));
                Player.channel = '';
            }else{
                Notifier.logError("Нет канала с таким номером.")
            }
        }, 2000);
        
    },
    handleKey: function(code) {
        switch (code) {
            case tvKey.KEY_1:
                Player.hidePanels();
                Player.getChannelByNumber(1);
                break;
            case tvKey.KEY_2:
                Player.hidePanels();
                Player.getChannelByNumber(2);
                break;
            case tvKey.KEY_3:
                Player.hidePanels();
                Player.getChannelByNumber(3);
                break;
            case tvKey.KEY_4:
                Player.hidePanels();
                Player.getChannelByNumber(4);
                break;
            case tvKey.KEY_5:
                Player.hidePanels();
                Player.getChannelByNumber(5);
                break;
            case tvKey.KEY_6:
                Player.hidePanels();
                Player.getChannelByNumber(6);
                break;
            case tvKey.KEY_7:
                Player.hidePanels();
                Player.getChannelByNumber(7);
                break;
            case tvKey.KEY_8:
                Player.hidePanels();
                Player.getChannelByNumber(8);
                break;
            case tvKey.KEY_9:
                Player.hidePanels();
                Player.getChannelByNumber(9);
                break;
            case tvKey.KEY_0:
                Player.hidePanels();
                Player.getChannelByNumber(0);
                break;
            case tvKey.KEY_LEFT:
            case tvKey.KEY_RIGHT:
                break;
            case tvKey.KEY_UP:
                Player.playChannel(Channel.getNextFavouriteChannel(), 0);
                break;
            case tvKey.KEY_DOWN:
                Player.playChannel(Channel.getPrevFavouriteChannel(), 0);
                break;
            case tvKey.KEY_RETURN: // Вернуться в Smart HUB
                widgetAPI.sendReturnEvent();
                break;
            case tvKey.KEY_ENTER:
                this.hidePanels();
                Menu.showAndHideMenu();
                break;
            case tvKey.KEY_INFO: // Показать инфо панели
                if (this.panelsShown) {
                    this.hidePanels();
                } else {
                    this.showPanels();
                }
                break;
            case tvKey.KEY_CH_UP: // Воспроизвести следующий канал
                Player.playChannel(Channel.getNextChannel());
                break;
            case tvKey.KEY_CH_DOWN: // Воспроизвести предыдущий канал
                Player.playChannel(Channel.getPrevChannel());
                break;
            case tvKey.KEY_PRECH: // Вернуться к предыдущему каналу
                if(Channel.prev_channel_id !== -1){
                    Channel.setCurrentChannelId(Channel.prev_channel_id);
                    Player.playChannel(Channel.getChannel(Channel.curr_channel_id));
                }
                break;
            case tvKey.KEY_YELLOW:
                if(Player.canChangeVideoregime){
                    clearTimeout(Player.videoregimeTimeout);
                    if(Player.currentWindowType == 'fullscreen'){
                        Player.currentWindowType = 'original';
                        Core.innerHTML(Core.getById('videoregime'), "Original");
                        if(Player.currentChannel.ratio == '15:11'){
                            Player.setPlayerVideoSize(112, 0, 736, 540);
                        }else{
                            Player.setPlayerVideoSize(120, 0, 720, 540);
                        }
                    }else{
                        Player.currentWindowType = 'fullscreen';
                        Core.innerHTML(Core.getById('videoregime'), "Full screen");
                        Player.setPlayerVideoSize(0, 0, 960, 540);
                    }
                    Core.showElement(Core.getById('videoregime'));
                    Player.videoregimeTimeout = setTimeout(function(){
                        Channel.setAspect(Player.currentChannel.id, Player.currentWindowType);
                        Core.hideElement(Core.getById('videoregime'));
                    }, 3000);
                }
                break;
            case tvKey.KEY_PAUSE:
            case tvKey.KEY_PLAY:
                if(Player.realTimePaused){
                    Player.realTimePaused = false;
                    Player.plugin.Execute('Resume');
                }else{
                    Player.realTimePaused = true;
                    Player.plugin.Execute('Pause');
                }
                break;
            case tvKey.KEY_BLUE:
                if(Player.countAudioTracks > 1){
                    clearTimeout(Player.audiotrackTimeout);
                    if(Player.currentStreamId < Player.countAudioTracks-1){
                        Player.currentStreamId++;
                    }else{
                        Player.currentStreamId = 0;
                    }
                    Player.plugin.Execute('SetStreamID',1, Player.currentStreamId);
                    Core.innerHTML(Core.getById('audiotrack'), "Звуковая дорожка: "+Player.currentStreamId);
                    Core.showElement(Core.getById('audiotrack'));
                    Player.audiotrackTimeout = setTimeout(function(){
                        Core.hideElement(Core.getById('audiotrack'));
                    }, 5000);
                }
                break;
        }
    },
    ConnectionFailed: function(){
//        Notifier.logError("ConnectionFailed")
    },
    NetworkDisconnected: function(){
//        Notifier.logError("NetworkDisconnected")
        Player.stop();
        Player.reopenInterval = setTimeout(Player.rePlayChannel, 10000);
    },
    RenderError: function(){
//        Notifier.logError("RenderError")
    },
    StreamNotFound: function(){
//        Notifier.logError("StreamNotFound")
    },
    AuthenticationFailed: function(){
//        Notifier.logError("AuthenticationFailed")
    },
    OnStreamInfoReady: function(){
        Player.currentStreamId = 0;
        Player.countAudioTracks = Player.plugin.Execute('GetTotalNumOfStreamID',1);
        
        if(Player.countAudioTracks > 1){
            Core.removeClass(Core.getById('playerBlueButtonPanel'), 'unshown');
        }else{
            Core.addClass(Core.getById('playerBlueButtonPanel'), 'unshown');
        }
        
//        Player.audioTracksInfo = [];
//        for (var i = 0; i < Player.countAudioTracks; i++) {
////            Player.audioTracksInfo.push(Player.plugin.Execute('GetStreamLanguageInfo',1,i));
////            Player.audioTracksInfo.push("'"+Player.plugin.Execute('GetStreamLanguage',1,i)+"'");
////            Player.audioTracksInfo.push('"'+Player.plugin.Execute('GetStreamLanguageCode',1,i)+'"');
//            alert('Audio lang ('+i+') lang: ' + Player.plugin.Execute('GetStreamLanguageInfo',1,i));
//        }
////        Notifier.logError("Player.plugin.audioTracksInfo = "+Player.audioTracksInfo.join(", "));
    }
};